package com.mdd.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yulichang.query.MPJQueryWrapper;
import com.mdd.admin.service.IArticleService;
import com.mdd.admin.validate.article.ArticleCreateValidate;
import com.mdd.admin.validate.article.ArticleSearchValidate;
import com.mdd.admin.validate.article.ArticleUpdateValidate;
import com.mdd.admin.validate.commons.PageValidate;
import com.mdd.admin.vo.article.ArticleDetailVo;
import com.mdd.admin.vo.article.ArticleListedVo;
import com.mdd.common.config.GlobalConfig;
import com.mdd.common.core.PageResult;
import com.mdd.common.entity.article.Article;
import com.mdd.common.mapper.article.ArticleCategoryMapper;
import com.mdd.common.mapper.article.ArticleMapper;
import com.mdd.common.util.StringUtils;
import com.mdd.common.util.TimeUtils;
import com.mdd.common.util.UrlUtils;
import java.util.Arrays;
import java.util.Iterator;
import javax.annotation.Resource;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

@Service
public class ArticleServiceImpl implements IArticleService {
   @Resource
   ArticleMapper articleMapper;
   @Resource
   ArticleCategoryMapper articleCategoryMapper;

   public PageResult<ArticleListedVo> list(PageValidate pageValidate, ArticleSearchValidate searchValidate) {
      Integer pageNo = pageValidate.getPageNo();
      Integer pageSize = pageValidate.getPageSize();
      MPJQueryWrapper<Article> mpjQueryWrapper = (MPJQueryWrapper)((MPJQueryWrapper)((MPJQueryWrapper)(new MPJQueryWrapper()).selectAll(Article.class).select(new String[]{"ac.name as category"}).innerJoin("?_article_category ac ON ac.id=t.cid".replace("?_", GlobalConfig.tablePrefix))).eq("t.is_delete", 0)).orderByDesc(Arrays.asList("t.sort", "t.id"));
      this.articleMapper.setSearch(mpjQueryWrapper, searchValidate, new String[]{"like:title@t.title:str", "=:cid@t.cid:int", "=:isShow@t.is_show:int", "datetime:startTime-endTime@t.create_time:str"});
      IPage<ArticleListedVo> iPage = this.articleMapper.selectJoinPage(new Page((long)pageNo, (long)pageSize), ArticleListedVo.class, mpjQueryWrapper);
      Iterator var7 = iPage.getRecords().iterator();

      while(var7.hasNext()) {
         ArticleListedVo vo = (ArticleListedVo)var7.next();
         vo.setImage(UrlUtils.toAbsoluteUrl(vo.getImage()));
         vo.setCreateTime(TimeUtils.timestampToDate(vo.getCreateTime()));
         vo.setUpdateTime(TimeUtils.timestampToDate(vo.getUpdateTime()));
      }

      return PageResult.iPageHandle(iPage);
   }

   public ArticleDetailVo detail(Integer id) {
      Article model = (Article)this.articleMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).select(Article.class, (info) -> {
         return !info.getColumn().equals("is_delete") && !info.getColumn().equals("delete_time");
      }).eq("id", id)).eq("is_delete", 0));
      Assert.notNull(model, "文章不存在");
      ArticleDetailVo vo = new ArticleDetailVo();
      BeanUtils.copyProperties(model, vo);
      vo.setContent(StringUtils.isNull(model.getContent()) ? "" : model.getContent());
      vo.setImage(UrlUtils.toAbsoluteUrl(model.getImage()));
      vo.setCreateTime(TimeUtils.timestampToDate(model.getCreateTime()));
      vo.setUpdateTime(TimeUtils.timestampToDate(model.getUpdateTime()));
      return vo;
   }

   public void add(ArticleCreateValidate createValidate) {
      Article model = new Article();
      model.setCid(createValidate.getCid());
      model.setTitle(createValidate.getTitle());
      model.setImage(UrlUtils.toRelativeUrl(createValidate.getImage()));
      model.setIntro(createValidate.getIntro());
      model.setContent(createValidate.getContent());
      model.setSummary(createValidate.getSummary());
      model.setSort(createValidate.getSort());
      model.setIsShow(createValidate.getIsShow());
      model.setAuthor(createValidate.getAuthor());
      model.setVisit(createValidate.getVisit());
      model.setCreateTime(TimeUtils.timestamp());
      model.setUpdateTime(TimeUtils.timestamp());
      this.articleMapper.insert(model);
   }

   public void edit(ArticleUpdateValidate updateValidate) {
      Article model = (Article)this.articleMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).eq("id", updateValidate.getId())).eq("is_delete", 0));
      Assert.notNull(model, "文章不存在!");
      Assert.notNull(this.articleCategoryMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).eq("id", updateValidate.getCid())).eq("is_delete", 0)), "分类不存在");
      model.setCid(updateValidate.getCid());
      model.setTitle(updateValidate.getTitle());
      model.setImage(UrlUtils.toRelativeUrl(updateValidate.getImage()));
      model.setIntro(updateValidate.getIntro());
      model.setContent(updateValidate.getContent());
      model.setSummary(updateValidate.getSummary());
      model.setVisit(updateValidate.getVisit());
      model.setIsShow(updateValidate.getIsShow());
      model.setAuthor(updateValidate.getAuthor());
      model.setSort(updateValidate.getSort());
      model.setUpdateTime(TimeUtils.timestamp());
      this.articleMapper.updateById(model);
   }

   public void del(Integer id) {
      Article article = (Article)this.articleMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id,is_show"}).eq("id", id)).eq("is_delete", 0));
      Assert.notNull(article, "文章不存在!");
      article.setIsDelete(1);
      article.setDeleteTime(TimeUtils.timestamp());
      this.articleMapper.updateById(article);
   }

   public void change(Integer id) {
      Article article = (Article)this.articleMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).select(new String[]{"id,is_show"}).eq("id", id)).eq("is_delete", 0));
      Assert.notNull(article, "文章不存在!");
      article.setIsShow(article.getIsShow() == 0 ? 1 : 0);
      article.setUpdateTime(TimeUtils.timestamp());
      this.articleMapper.updateById(article);
   }
}
